//-------GFFileSystem.h------------------------------//
//  Global functions for file handling
#ifndef GFFILESYSTEM_H
#define GFFILESYSTEM_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef WANT_SDL
	#define WIN32_LEAN_AND_MEAN		// Selten benutzte Teile der Windows-Header nicht einbinden

	#ifdef _WINDOWS_
	#include <windows.h>
	#else
	#include "afxwin.h"
	#endif
#else
	typedef const char* LPCTSTR;
	typedef char* LPSTR;
#ifndef NULL
	#define NULL (0L)
#endif
	#define DWORD	unsigned long
	typedef unsigned char   BYTE;
	typedef unsigned short  WORD;

	#define TRUE	1
	#define	FALSE	0
#endif

#include <string.h> // strcpy, strcat

#include <stdio.h> // sprintf - wsprintf %g is different!?

#ifndef WIN32
   #include <stdlib.h>
   #include <sys/types.h>
   #include <sys/stat.h>
   #include <unistd.h>
   #include <dirent.h> // opendir/readdir/closedir
   #include <sys/stat.h> // Is Directory
#else
//	#define sprintf wsprintf // %g differs
#endif

class GFFileSystem
{
public:
enum SEEK_FROM
{
	start = 0,
	current,
	end
};

	GFFileSystem()	{m_file=NULL;}
	~GFFileSystem()	{Close();}
static bool SetCurrentDir(const char* cdir)
{
	#ifdef WIN32
	return ::SetCurrentDirectory(cdir) ? true : false;
	#else
	return (0 == chdir(cdir)) ? true : false; // _ removed?
	#endif
}

static void GetCurrentDir(char* cdir)
{
	#ifdef WIN32
		::GetCurrentDirectory(MAX_PATH, cdir);
	#else
		if(getcwd(cdir, MAX_PATH) == NULL)
		{
			cdir[0] = '\0';
		}
	#endif
}

static void GetExePath(char* cexepathname)
{
	#ifndef WIN32
		char fullpath[1024], *exepath;
		if ((exepath = getenv("_")) && realpath(exepath, fullpath))// resolve ./ ../
		{
			strcpy(cexepathname, fullpath);
		}
		else cexepathname[0]='\0';
	#else
		::GetModuleFileName(NULL, cexepathname, MAX_PATH);
	#endif
}


#ifndef WIN32
/* Case-insensitive file open.  As you can see, this is an expensive
   call.  We don't regard it as our main strategy for dealing with
   case-insensitivity.  Returns errno code or 0 for success.  Puts the
   new file in *FP.  NAME and MODE are as for fopen.  If PATHP is not
   NULL, then put a malloc'd string containing the pathname as found
   into *PATHP.  *PATHP is only set if the return value is 0.

   Might be cleaner to separate the file finding (which just gives
   *PATHP) from the file opening (which the caller can do).  For one
   thing, might make it easier to know whether to put NAME or *PATHP
   into error messages.  */
FILE* fopen_case (const char* name, const char* mode)
{
	FILE* fp=fopen(name, mode);
	if(fp) return fp;
	struct dirent *dp;
    DIR *dirp=NULL;
    char *dir=NULL;
    char *fname=NULL;
    char *found_name=NULL;

    // Separate NAME into directory DIR and filename within the directory FNAME.
    dir = strdup (name);
    fname = strrchr (dir, '/');
	if (fname == NULL)
	{
		char temp[1024]; strcpy(temp, "./"); strcat(temp, name);
		free(dir);
		dir=strdup(temp);
		fname=dir+2;
		dir[1]='\0';
	}
	else
	{
		//	error (1, 0, "internal error: relative pathname in fopen_case");
	    *fname++ = '\0';
	}
    found_name = NULL;
	dirp = opendir(dir);
    if (dirp == NULL)
    {
	    printf("cannot read directory %s - watch case sensitivity for directory names!\n", dir);
		goto err;
    }
    while ((dp = readdir (dirp)) != NULL)
    {
		// first match is taken
		if (strcasecmp(dp->d_name, fname) == 0)
		{
			found_name = strdup(dp->d_name);
			break;
		}
    }
    closedir (dirp);

    if (found_name == NULL) goto err;

	char *p;
	/* Copy the found name back into DIR.  We are assuming that
	   found_name is the same length as fname, which is true as
	   long as the above code is just ignoring case and not other
	   aspects of filename syntax.  */
	if(dir[0]!='.')
	{
		p = dir + strlen (dir);
		*p++ = '/';
	}
	else
		p=dir;
	strcpy (p, found_name);
	fp = fopen (dir, mode);
	free (dir);
	free (found_name);
	return fp;

err:
	free(dir);
	return NULL;
}
#else
	#define fopen_case	fopen
#endif


// ------------------------------------------------------------- //
bool GFFileSystem::OpenIn(LPCTSTR file)
// ------------------------------------------------------------- //
{
	if (m_file) this->Close();
#ifdef WIN32
	m_file = CreateFile(file, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (m_file==INVALID_HANDLE_VALUE) m_file=NULL;
#else
	m_file=fopen_case(file, "rb");
#endif
	return m_file ? true : false;
}

// ------------------------------------------------------------- //
bool GFFileSystem::OpenOut(LPCTSTR file)
// ------------------------------------------------------------- //
{
	if (m_file) this->Close();
#ifdef WIN32
	m_file = CreateFile(file, GENERIC_WRITE, NULL, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	if (m_file==INVALID_HANDLE_VALUE) m_file=NULL;
#else
	m_file=fopen_case(file, "wb");
#endif
	return m_file ? true : false;
}

// ------------------------------------------------------------- //
bool GFFileSystem::OpenAppend(LPCTSTR file)
// ------------------------------------------------------------- //
{
	if (m_file) this->Close();
#ifdef WIN32
	m_file = CreateFile(file, GENERIC_WRITE, NULL, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	if (m_file==INVALID_HANDLE_VALUE) m_file=NULL;
		SetFilePointer(m_file, 0, NULL, FILE_END);
#else
	m_file=fopen_case(file, "ab");
#endif
	return m_file ? true : false;
}

// ------------------------------------------------------------- //
// ------------------------------------------------------------- //
bool GFFileSystem::SeekTo(long pos, SEEK_FROM sf=start)
{
	if (!m_file)
		return false;
#ifdef WIN32
	DWORD dwsf = FILE_BEGIN;
	if (sf == current)
		dwsf = FILE_CURRENT;
	else if (sf==end)
		dwsf = FILE_END;

	#ifndef INVALID_SET_FILE_POINTER
	#define INVALID_SET_FILE_POINTER ((DWORD)0xFFFFFFFF)
	#endif
	return (INVALID_SET_FILE_POINTER == SetFilePointer(m_file, (LONG)pos, 0L, dwsf)) ? false : true;
#else
	int isf = 0;
	switch(sf)
	{
	case GFFileSystem::current:
		isf=SEEK_CUR; break;
	case GFFileSystem::end:
		isf=SEEK_END; break;
	default:
		isf=SEEK_SET;
	}
	return (fseek(m_file, (long)pos, isf) == 0) ? true : false;
#endif
}

// ------------------------------------------------------------- //
// ------------------------------------------------------------- //
void GFFileSystem::Close()
{
#ifdef WIN32
	if (m_file)
		CloseHandle(m_file);
#else
	if (m_file) fclose(m_file);
#endif
	m_file = NULL;
}
// ------------------------------------------------------------- //

// ------------------------------------------------------------- //
bool GFFileSystem::Write(void* buffer, int bytes)
// ------------------------------------------------------------- //
{
#ifdef WIN32
	DWORD	nread;
	if (m_file)
	{
		WriteFile(m_file, buffer, bytes, &nread, NULL);
		return (nread == (DWORD)bytes) ? true : false;
	}
#else
	if (m_file)
		if (fwrite(buffer, bytes, 1, m_file) == 1) return true;
#endif
return false;
}

// ------------------------------------------------------------- //
// ------------------------------------------------------------- //
bool GFFileSystem::WriteLine(const char* line)
{
	Write((void*)line, (int)strlen(line));
	return Write((void*)"\r\n", 2);
}

// ------------------------------------------------------------- //
// ------------------------------------------------------------- //
bool GFFileSystem::Read(void* buffer, int bytes)
{
#ifdef WIN32
	DWORD	nread;
	if (m_file)
	{
		ReadFile(m_file, buffer, bytes, &nread, NULL);
		return (nread == (DWORD)bytes) ? true : false;
	}
#else
	if (m_file)
		if (fread(buffer, bytes, 1, m_file)==1) return true;
#endif
return false;
}

// ------------------------------------------------------------- //
// ------------------------------------------------------------- //
bool ReadLine(void* buffer) // Read to next '\n' truncating '\r'
{
char*	pBuffer = (char*)buffer;
	if (!Read(pBuffer, 1))
		return 0;
	do
	{
		if (*pBuffer == '\n')
		{
			*pBuffer = '\0';
				if (pBuffer>buffer && *(pBuffer-1) == '\r')
					*(pBuffer-1)='\0';
			break;
		}
		pBuffer++;
	} while(Read(pBuffer, 1));
	*pBuffer ='\0';
return true;
}

void GFFileSystem::operator << (int& num) {this->Write(&num, sizeof(num));}
void GFFileSystem::operator << (float& num) {this->Write(&num, sizeof(num));}
void GFFileSystem::operator << (double& num) {this->Write(&num, sizeof(num));}
void GFFileSystem::operator << (DWORD& num) {this->Write(&num, sizeof(num));}
void GFFileSystem::operator << (long& num) {this->Write(&num, sizeof(num));}
void GFFileSystem::operator << (char& num) {this->Write(&num, sizeof(num));}

void GFFileSystem::operator >> (int& num) {this->Read(&num, sizeof(num));}
void GFFileSystem::operator >> (float& num) {this->Read(&num, sizeof(num));}
void GFFileSystem::operator >> (double& num) {this->Read(&num, sizeof(num));}
void GFFileSystem::operator >> (DWORD& num) {this->Read(&num, sizeof(num));}
void GFFileSystem::operator >> (long& num) {this->Read(&num, sizeof(num));}
void GFFileSystem::operator >> (char& num) {this->Read(&num, sizeof(num));}


	//---------------------------------------------------------------//
	//---------------------------------------------------------------//
	static void Delete(const char* filename)
	{
		#ifndef WANT_SDL
		::DeleteFile(filename);
		#else
		remove(filename);
		#endif
	}

	//---------------------------------------------------------------//
	//---------------------------------------------------------------//
	static void GetTempPath(long size, char* filename)
	{
		#ifdef WIN32
		::GetTempPath(size,filename); // C:\Temp\...
		#else
		strcpy(filename, "/tmp/");
		#endif

	}
	//---------------------------------------------------------------//
	//---------------------------------------------------------------//
	static void DeleteDebug()
	{
		char filename[MAX_PATH];
		GFFileSystem::GetTempPath(MAX_PATH, filename);
		strcat(filename, "ErrorLog.txt");
		::GFFileSystem::Delete(filename);
	}

	//---------------------------------------------------------------//
	//---------------------------------------------------------------//
	static void WriteDebug(LPCTSTR text)
	{
		char filename[MAX_PATH];
		GFFileSystem::GetTempPath(MAX_PATH, filename);
		GFFileSystem	fs;
		strcat(filename, "ErrorLog.txt");
		if(fs.OpenAppend(filename))
		{
			fs.WriteLine(text);
			fs.Close();
		}
		//FILE* file = fopen_case(filename, "a");
		// if (file){fputs(text, file); fputs("\r\n", file); fclose(file);}
	}

	//---------------------------------------------------------------//
	//---------------------------------------------------------------//
	static void WriteDebug(double number)
	{
		char num[256];
		sprintf(num, "%g", number);
		WriteDebug(num);
	}

	//---------------------------------------------------------------//
	//---------------------------------------------------------------//
	static bool DoesFileExist(LPCTSTR filename)
	{
		GFFileSystem	fs;
		if (fs.OpenIn(filename))
		{
			fs.Close();
			return true;
		}
		return false;
	}

protected:
#ifdef WIN32
	HANDLE	m_file;
#else
	FILE*	m_file;
#endif
};



#define g_deldebug()		GFFileSystem::DeleteDebug();
#define g_debug(a)			GFFileSystem::WriteDebug(a)
#define g_FileExist(a)		GFFileSystem::DoesFileExist(a)

#endif // GFFILESYSTEM
