#if !defined(AFX_MESHOBJECT_H__A54DCD42_E1B4_11D5_B6EB_003042FFFD01__INCLUDED_)
#define AFX_MESHOBJECT_H__A54DCD42_E1B4_11D5_B6EB_003042FFFD01__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "GFqueue.h"
#include "VectorMath.h"

#define NUMVERTEXNORMALS	162
extern float gAvertexnormals[NUMVERTEXNORMALS][3];


#define	COL_NEW_TRI		0xFFFFFFFF // COL_NEW_TRI=Triangles COL_NEW_STRIP=TriStrips COL_NEW_FAN=TriFan
#define COL_NEW_STRIP	0xFFFFFFFE
#define COL_NEW_FAN		0xFFFFFFFD
#define COL_NEW_ANY		0xFFFFFFFC // Bigger than this-> a new part starts


// Data
struct POLY_NODE // KEEP this order for load/save/memcpy operations!!
{
	float			pos[3];
	float			normal[3];
	float			tex[2];
	// Additional (not saved) data
	float			bump_light[3]; // offset vertex glColor
	float			tangent[3];
	float			binormal[3];
	unsigned long	col;
};

struct XPOLY_NODE
{
	unsigned char	pos[3];
	unsigned char	inormal, itangent, ibinormal;
	float			tangent[3];
	float			binormal[3];
	unsigned char	icol; // Index in color table
};

class POLY_FRAME
{
public:
	Queue <POLY_NODE>	nodes;		// The Nodes of the mesh
	Queue <POLY_NODE>	triangles;	// For fast VBO rendering. Every 3 nodes are one triangle.
	Queue <XPOLY_NODE>	xnodes;
	float				move[3];
	float				scale[3];
	float				max_radius;	// For pre-calc of collisions
	POLY_FRAME();
	~POLY_FRAME();
	POLY_FRAME& operator = (POLY_FRAME& f);
};

class MeshObject
{
public:
	// Data
	Queue <POLY_FRAME>	m_Frames; // The Animation Frames
	Queue <POLY_NODE>	m_tri_animation; // triangles for interoplated animation
	Queue <float>		m_Texture[2]; // The Texture coords
	Queue <unsigned long>m_ColTable;// Colors
	long				m_HasNormals; // Does the object already have calculated normals?

public:
	// Functions
	MeshObject();
	~MeshObject();
	void				Clear(bool free_mem);

	inline int			NumFrames()				{return m_Frames.Size();}
	inline int			NumNodes(int frame)		{return m_Frames.Size()<=(unsigned int)frame ? 0 : m_Frames.GetAt(frame)->xnodes.Size();}
	inline void			PushNode(int frame, float pos[3], float tex[2], unsigned long col)
						{
						POLY_NODE pn;
							pn.pos[0]=pos[0]; pn.pos[1]=pos[1]; pn.pos[2]=pos[2];
							pn.tex[0]=tex[0]; pn.tex[1]=tex[1]; pn.col=col;
							m_Frames.GetAt(frame)->nodes.Push(pn);
						}

	void				Compress(void);
	inline void			Uncompress(int frame, int index, POLY_NODE* pNode)
						{
						register POLY_FRAME*	pFrame;
						register XPOLY_NODE*	pXNode;
						register int i;
							pFrame=m_Frames.GetAt(frame);
							pXNode=pFrame->xnodes.GetAt(index);
							for (i=0; i<3; i++)
							{
								pNode->pos[i]=((float)pXNode->pos[i]-pFrame->move[i])*pFrame->scale[i];
								pNode->normal  [i]=gAvertexnormals[pXNode->inormal  ][i];
								
								pNode->tangent [i]=gAvertexnormals[pXNode->itangent ][i];
								pNode->binormal[i]=gAvertexnormals[pXNode->ibinormal][i];
							}

							pNode->col=*m_ColTable.GetAt(pXNode->icol);
							pNode->tex[0]=   *this->m_Texture[0].GetAt(index);
							pNode->tex[1]= - *this->m_Texture[1].GetAt(index); // texture flipped due to grab sprite compatibility
						}

	bool				Save(const char* file);
	bool				Load(const char* file);
};

#endif